from sqlalchemy import *
from sqlalchemy.orm import *
from sqlalchemy.ext.declarative import declared_attr
from ..types import BrawlhallaRank, BrawlhallaTier, BrawlhallaMetal


class BrawlhallaDuo:
    __tablename__ = "brawlhalladuos"

    @declared_attr
    def id_one(self):
        return Column(Integer, ForeignKey("brawlhalla.brawlhalla_id"), primary_key=True)

    @declared_attr
    def id_two(self):
        return Column(Integer, ForeignKey("brawlhalla.brawlhalla_id"), primary_key=True)

    @declared_attr
    def one(self):
        return relationship("Brawlhalla", foreign_keys=self.id_one, backref=backref("_duos_one"))

    @declared_attr
    def two(self):
        return relationship("Brawlhalla", foreign_keys=self.id_two, backref=backref("_duos_two"))

    @declared_attr
    def rating_2v2(self):
        return Column(Integer)

    @declared_attr
    def tier_2v2(self):
        return Column(Enum(BrawlhallaTier))

    @declared_attr
    def metal_2v2(self):
        return Column(Enum(BrawlhallaMetal))

    @property
    def rank_2v2(self):
        return BrawlhallaRank(metal=self.metal_2v2, tier=self.tier_2v2)

    @rank_2v2.setter
    def rank_2v2(self, value):
        if not isinstance(value, BrawlhallaRank):
            raise TypeError("rank_1v1 can only be set to BrawlhallaRank values.")
        self.metal_2v2 = value.metal
        self.tier_2v2 = value.tier

    def other(self, bh):
        if bh == self.one:
            return self.two
        elif bh == self.two:
            return self.one
        else:
            raise ValueError("Argument is unrelated to this duo.")

    def __repr__(self):
        return f"<BrawlhallaDuo {self.id_one} & {self.id_two}>"



from typing import *
import datetime
import re
import dateparser
import typing
import random
import enum
import asyncio as aio
from telegram import Bot as PTBBot
from telegram import Message as PTBMessage
from telegram import InlineKeyboardMarkup as InKM
from telegram import InlineKeyboardButton as InKB
from telegram.error import TelegramError
import royalnet.commands as rc
from royalnet.serf.telegram import TelegramSerf as TelegramBot
from royalnet.serf.telegram import escape as telegram_escape
from royalnet.utils import asyncify, sleep_until, sentry_async_wrap
from royalnet.backpack.tables import User
from ..tables import MMEvent, MMResponse, FiorygiTransaction
from ..types import MMChoice, MMInterfaceDataTelegram


class Interrupts(enum.Enum):
    TIME_RAN_OUT = enum.auto()
    MANUAL_START = enum.auto()
    MANUAL_DELETE = enum.auto()


class MatchmakingCommand(rc.Command):
    name: str = "matchmaking"

    description: str = "Cerca persone per una partita a qualcosa!"

    syntax: str = "[ {ora} ] {nome}\n[descrizione]"

    aliases = ["mm", "lfg"]

    tables = {MMEvent, MMResponse}

    def __init__(self, interface: rc.CommandInterface):
        super().__init__(interface)
        # Find all relevant MMEvents and run them
        session = self.alchemy.Session()
        mmevents = (
            session
            .query(self.alchemy.get(MMEvent))
            .filter(self.alchemy.get(MMEvent).interface == self.interface.name,
                    self.alchemy.get(MMEvent).datetime > datetime.datetime.now(),
                    self.alchemy.get(MMEvent).interrupted == False)
            .all()
        )
        self.tasks_created = {}
        self.queue: Dict[int, aio.queues.Queue] = {}
        for mmevent in mmevents:
            task = self.interface.loop.create_task(self._run_mmevent(mmevent.mmid))
            self.tasks_created[mmevent.mmid] = task

    async def run(self, args: rc.CommandArgs, data: rc.CommandData) -> None:
        # Create a new MMEvent and run it
        if self.interface.name != "telegram":
            raise rc.UnsupportedError(f"{self.interface.prefix}matchmaking funziona solo su Telegram. Per ora.")
        author = await data.get_author(error_if_none=True)

        try:
            timestring, title, description = args.match(r"\[\s*([^]]+)\s*]\s*([^\n]+)\s*\n?\s*(.+)?\s*", re.DOTALL)
        except rc.InvalidInputError:
            timestring, title, description = args.match(r"\s*(.+?)\s*\n\s*([^\n]+)\s*\n?\s*(.+)?\s*", re.DOTALL)
        try:
            dt: typing.Optional[datetime.datetime] = dateparser.parse(timestring, settings={
                "PREFER_DATES_FROM": "future"
            })
        except OverflowError:
            dt = None
        if dt is None:
            raise rc.InvalidInputError("La data che hai specificato non è valida.")
        if dt <= datetime.datetime.now():
            raise rc.InvalidInputError("La data che hai specificato è nel passato.")
        if dt - datetime.datetime.now() >= datetime.timedelta(days=366):
            raise rc.InvalidInputError("Hai specificato una data tra più di un anno!\n"
                                       "Se volevi scrivere un'orario, ricordati che le ore sono separate da due punti"
                                       " (:) e non da punto semplice!")
        mmevent: MMEvent = self.alchemy.get(MMEvent)(creator=author,
                                                     datetime=dt,
                                                     title=title,
                                                     description=description,
                                                     interface=self.interface.name)
        data.session.add(mmevent)
        await data.session_commit()
        self.loop.create_task(self._run_mmevent(mmevent.mmid))
        await data.reply(f"🚩 Matchmaking creato!")

    _mmchoice_values = {
        MMChoice.YES: 4,
        MMChoice.LATE_SHORT: 3,
        MMChoice.LATE_MEDIUM: 2,
        MMChoice.LATE_LONG: 1,
        MMChoice.MAYBE: 0,
        MMChoice.NO: -1
    }

    def _gen_mm_message(self, mmevent: MMEvent) -> str:
        text = f"🚩 [{mmevent.datetime.strftime('%Y-%m-%d %H:%M')}] [b]{mmevent.title}[/b]\n"
        if mmevent.description:
            text += f"{mmevent.description}\n"
        text += "\n"
        for response in sorted(mmevent.responses, key=lambda r: -self._mmchoice_values[r.choice]):
            response: MMResponse

            if response.choice == MMChoice.LATE_SHORT:
                td = mmevent.datetime + datetime.timedelta(minutes=10)
                time_text = f" [{td.strftime('%H:%M')}]"
            elif response.choice == MMChoice.LATE_SHORT:
                td = mmevent.datetime + datetime.timedelta(minutes=30)
                time_text = f" [{td.strftime('%H:%M')}]"
            elif response.choice == MMChoice.LATE_SHORT:
                td = mmevent.datetime + datetime.timedelta(minutes=60)
                time_text = f" [{td.strftime('%H:%M')}+]"
            else:
                time_text = ""

            creator_crown = " 👑" if response.user == mmevent.creator else ""

            text += f"{response.choice.value} {response.user}{time_text}{creator_crown}\n"
        return text

    @staticmethod
    def _gen_telegram_keyboard(mmevent: MMEvent):
        if mmevent.datetime <= datetime.datetime.now():
            return None
        return InKM([
            [
                InKB(f"{MMChoice.YES.value} Ci sarò!",
                     callback_data=f"mm{mmevent.mmid}_YES"),
                InKB(f"{MMChoice.MAYBE.value} Forse...",
                     callback_data=f"mm{mmevent.mmid}_MAYBE"),
                InKB(f"{MMChoice.NO.value} Non mi interessa.",
                     callback_data=f"mm{mmevent.mmid}_NO"),
            ],
            [
                InKB(f"{MMChoice.LATE_SHORT.value} 10 min",
                     callback_data=f"mm{mmevent.mmid}_LATE_SHORT"),
                InKB(f"{MMChoice.LATE_MEDIUM.value} 30 min",
                     callback_data=f"mm{mmevent.mmid}_LATE_MEDIUM"),
                InKB(f"{MMChoice.LATE_LONG.value} 60+ min",
                     callback_data=f"mm{mmevent.mmid}_LATE_LONG"),
            ],
            [
                InKB(f"🗑 Elimina",
                     callback_data=f"mm{mmevent.mmid}_DELETE"),
                InKB(f"🚩 Inizia",
                     callback_data=f"mm{mmevent.mmid}_START"),
            ]
        ])

    async def _update_telegram_mm_message(self, client: PTBBot, mmevent: MMEvent):
        try:
            await self.interface.serf.api_call(client.edit_message_text,
                                               chat_id=self.config["Matchmaking"]["mm_chat_id"],
                                               text=telegram_escape(self._gen_mm_message(mmevent)),
                                               message_id=mmevent.interface_data.message_id,
                                               parse_mode="HTML",
                                               disable_web_page_preview=True,
                                               reply_markup=self._gen_telegram_keyboard(mmevent))
        except TelegramError:
            pass

    def _gen_mm_telegram_callback(self, client: PTBBot, mmid: int, choice: MMChoice):
        async def callback(data: rc.CommandData):
            author = await data.get_author(error_if_none=True)
            mmevent: MMEvent = await asyncify(data.session.query(self.alchemy.get(MMEvent)).get, mmid)
            mmresponse: MMResponse = await asyncify(
                data.session.query(self.alchemy.get(MMResponse)).filter_by(user=author, mmevent=mmevent).one_or_none)
            if mmresponse is None:
                mmresponse = self.alchemy.get(MMResponse)(user=author, mmevent=mmevent, choice=choice)
                data.session.add(mmresponse)
                if random.randrange(14) == 0:
                    await FiorygiTransaction.spawn_fiorygi(data, author, 1, "aver risposto a un matchmaking")
            else:
                mmresponse.choice = choice
            await data.session_commit()
            await self._update_telegram_mm_message(client, mmevent)
            await data.reply(f"{choice.value} Messaggio ricevuto!")
        return callback

    def _gen_mm_telegram_delete(self, client, mmid: int):
        async def callback(data: rc.CommandData):
            author = await data.get_author(error_if_none=True)
            mmevent: MMEvent = await asyncify(data.session.query(self.alchemy.get(MMEvent)).get, mmid)
            if author != mmevent.creator:
                raise rc.UserError("Non sei il creatore di questo matchmaking!")
            await self.queue[mmid].put(Interrupts.MANUAL_DELETE)
            await data.reply(f"🗑 Evento eliminato!")
        return callback

    def _gen_mm_telegram_start(self, client, mmid: int):
        async def callback(data: rc.CommandData):
            author = await data.get_author(error_if_none=True)
            mmevent: MMEvent = await asyncify(data.session.query(self.alchemy.get(MMEvent)).get, mmid)
            if author != mmevent.creator:
                raise rc.UserError("Non sei il creatore di questo matchmaking!")
            await self.queue[mmid].put(Interrupts.MANUAL_START)
            await data.reply(f"🚩 Evento avviato!")
        return callback

    async def _set_event_after(self, mmid: int, dt: datetime.datetime):
        await sleep_until(dt)
        if mmid in self.queue:
            await self.queue[mmid].put(Interrupts.TIME_RAN_OUT)

    def _gen_event_start_message(self, mmevent: MMEvent):
        text = f"🚩 L'evento [b]{mmevent.title}[/b] è iniziato!\n\n"
        for response in sorted(mmevent.responses, key=lambda r: -self._mmchoice_values[r.choice]):
            response: MMResponse
            text += f"{response.choice.value} {response.user}\n"
        return text

    @staticmethod
    def _gen_unauth_message(user: User):
        return f"⚠️ Non sono autorizzato a mandare messaggi a [b]{user.username}[/b]!\n" \
               f"{user.telegram[0].mention()}, apri una chat privata con me e mandami un messaggio!"

    @sentry_async_wrap()
    async def _run_mmevent(self, mmid: int):
        """Run a MMEvent."""
        # Create the event in the dict
        self.queue[mmid] = aio.Queue()
        # Open a new Alchemy Session
        session = self.alchemy.Session()
        # Find the MMEvent with the current session
        mmevent: MMEvent = await asyncify(session.query(self.alchemy.get(MMEvent)).get, mmid)
        if mmevent is None:
            raise ValueError("Invalid mmid.")
        # Ensure the MMEvent hasn't already started
        if mmevent.datetime <= datetime.datetime.now():
            raise ValueError("MMEvent has already started.")
        # Ensure the MMEvent interface matches the current one
        if mmevent.interface != self.interface.name:
            raise ValueError("Invalid interface.")
        # If the matchmaking message hasn't been sent yet, do so now
        if mmevent.interface_data is None:
            if self.interface.name == "telegram":
                bot: TelegramBot = self.interface.serf
                client: PTBBot = bot.client
                # Send the keyboard
                message: PTBMessage = await self.interface.serf.api_call(
                    client.send_message,
                    chat_id=self.config["Matchmaking"]["mm_chat_id"],
                    text=telegram_escape(
                        self._gen_mm_message(mmevent)),
                    parse_mode="HTML",
                    disable_webpage_preview=True,
                    reply_markup=self._gen_telegram_keyboard(mmevent)
                )
                # Store message data in the interface data object
                mmevent.interface_data = MMInterfaceDataTelegram(chat_id=self.config["Matchmaking"]["mm_chat_id"],
                                                                 message_id=message.message_id)
                await asyncify(session.commit)
            else:
                raise rc.UnsupportedError()
        # Register handlers for the keyboard events
        if self.interface.name == "telegram":
            bot: TelegramBot = self.interface.serf
            client: PTBBot = bot.client
            bot.register_keyboard_key(f"mm{mmevent.mmid}_YES", key=rc.KeyboardKey(
                interface=self.interface,
                short=f"{MMChoice.YES.value}",
                text="Ci sarò!",
                callback=self._gen_mm_telegram_callback(client, mmid, MMChoice.YES)
            ))
            bot.register_keyboard_key(f"mm{mmevent.mmid}_LATE_SHORT", key=rc.KeyboardKey(
                interface=self.interface,
                short=f"{MMChoice.LATE_SHORT.value}",
                text="10 min",
                callback=self._gen_mm_telegram_callback(client, mmid, MMChoice.LATE_SHORT)
            ))
            bot.register_keyboard_key(f"mm{mmevent.mmid}_LATE_MEDIUM", key=rc.KeyboardKey(
                interface=self.interface,
                short=f"{MMChoice.LATE_MEDIUM.value}",
                text="30 min",
                callback=self._gen_mm_telegram_callback(client, mmid, MMChoice.LATE_MEDIUM)
            ))
            bot.register_keyboard_key(f"mm{mmevent.mmid}_LATE_LONG", key=rc.KeyboardKey(
                interface=self.interface,
                short=f"{MMChoice.LATE_LONG.value}",
                text="60 min",
                callback=self._gen_mm_telegram_callback(client, mmid, MMChoice.LATE_LONG)
            ))
            bot.register_keyboard_key(f"mm{mmevent.mmid}_MAYBE", key=rc.KeyboardKey(
                interface=self.interface,
                short=f"{MMChoice.MAYBE.value}",
                text="Forse...",
                callback=self._gen_mm_telegram_callback(client, mmid, MMChoice.MAYBE)
            ))
            bot.register_keyboard_key(f"mm{mmevent.mmid}_NO", key=rc.KeyboardKey(
                interface=self.interface,
                short=f"{MMChoice.NO.value}",
                text="Non mi interessa.",
                callback=self._gen_mm_telegram_callback(client, mmid, MMChoice.NO)
            ))
            bot.register_keyboard_key(f"mm{mmevent.mmid}_DELETE", key=rc.KeyboardKey(
                interface=self.interface,
                short=f"🗑",
                text="Elimina",
                callback=self._gen_mm_telegram_delete(client, mmid)
            ))
            bot.register_keyboard_key(f"mm{mmevent.mmid}_START", key=rc.KeyboardKey(
                interface=self.interface,
                short=f"🚩",
                text="Inizia",
                callback=self._gen_mm_telegram_start(client, mmid)
            ))
        else:
            raise rc.UnsupportedError()
        # Sleep until something interrupts
        self.loop.create_task(self._set_event_after(mmid, mmevent.datetime))
        interrupt = await self.queue[mmid].get()
        mmevent.interrupted = True
        await asyncify(session.commit)
        del self.queue[mmid]
        # Notify the positive answers of the event start
        if self.interface.name == "telegram":
            bot: TelegramBot = self.interface.serf
            client: PTBBot = bot.client
            bot.unregister_keyboard_key(f"mm{mmevent.mmid}_YES")
            bot.unregister_keyboard_key(f"mm{mmevent.mmid}_MAYBE")
            bot.unregister_keyboard_key(f"mm{mmevent.mmid}_LATE_SHORT")
            bot.unregister_keyboard_key(f"mm{mmevent.mmid}_LATE_MEDIUM")
            bot.unregister_keyboard_key(f"mm{mmevent.mmid}_LATE_LONG")
            bot.unregister_keyboard_key(f"mm{mmevent.mmid}_NO")
            bot.unregister_keyboard_key(f"mm{mmevent.mmid}_DELETE")
            bot.unregister_keyboard_key(f"mm{mmevent.mmid}_START")

            if interrupt == Interrupts.TIME_RAN_OUT or interrupt == Interrupts.MANUAL_START:
                await asyncify(client.send_message,
                               chat_id=self.config["Telegram"]["main_group_id"],
                               text=telegram_escape(self._gen_event_start_message(mmevent)),
                               parse_mode="HTML",
                               disable_webpage_preview=True)

                for response in mmevent.responses:
                    if response.choice == MMChoice.NO:
                        return
                    try:
                        await asyncify(client.send_message,
                                       chat_id=response.user.telegram[0].tg_id,
                                       text=telegram_escape(self._gen_event_start_message(mmevent)),
                                       parse_mode="HTML",
                                       disable_webpage_preview=True)
                    except TelegramError:
                        await self.interface.serf.api_call(
                            client.send_message,
                            chat_id=self.config["Telegram"]["main_group_id"],
                            text=telegram_escape(self._gen_unauth_message(response.user)),
                            parse_mode="HTML",
                            disable_webpage_preview=True
                        )

            elif interrupt == Interrupts.MANUAL_DELETE:
                await self.interface.serf.api_call(
                    client.send_message,
                    chat_id=self.config["Telegram"]["main_group_id"],
                    text=telegram_escape(f"🗑 L'evento [b]{mmevent.title}[/b] è stato annullato."),
                    parse_mode="HTML",
                    disable_webpage_preview=True
                )

            await self.interface.serf.api_call(client.delete_message,
                                               chat_id=mmevent.interface_data.chat_id,
                                               message_id=mmevent.interface_data.message_id)

        else:
            raise rc.UnsupportedError()
        # The end!
        await asyncify(session.close)
        del self.tasks_created[mmid]



from typing import *
from royalnet.commands import *
from royalnet.utils import *
from royalnet.backpack.tables import User, Alias
from sqlalchemy import func


class UserinfoCommand(Command):
    name: str = "userinfo"

    aliases = ["uinfo", "ui", "useri"]

    description: str = "Visualizza informazioni su un utente."

    syntax = "[username]"

    async def run(self, args: CommandArgs, data: CommandData) -> None:
        username = args.optional(0)
        if username is None:
            user: User = await data.get_author(error_if_none=True)
        else:
            found: Optional[User] = await Alias.find_user(self.alchemy, data.session, username)
            if not found:
                raise InvalidInputError("Utente non trovato.")
            else:
                user = found

        r = [
            f"ℹ️ [url=https://ryg.steffo.eu/#/user/{user.uid}]{user.username}[/url]",
            f"{user.role}",
            "",
        ]

        # Bios are a bit too long
        # if user.bio:
        #     r.append(f"{user.bio}")

        for account in user.telegram:
            r.append(f"{account}")

        for account in user.discord:
            r.append(f"{account}")

        for account in user.steam:
            r.append(f"{account}")
            if account.dota is not None:
                r.append(f"{account.dota}")
            if account.brawlhalla is not None:
                r.append(f"{account.brawlhalla}")

        for account in user.leagueoflegends:
            r.append(f"{account}")

        r.append("")

        r.append(f"Ha creato [b]{len(user.diario_created)}[/b] righe di "
                 f"[url=https://ryg.steffo.eu/#/diario]Diario[/url], e vi compare in"
                 f" [b]{len(user.diario_quoted)}[/b] righe.")

        r.append("")

        if user.trivia_score:
            r.append(f"Ha [b]{user.trivia_score.score:.0f}[/b] punti Trivia, avendo risposto correttamente a"
                     f" [b]{user.trivia_score.correct_answers}[/b] domande su"
                     f" [b]{user.trivia_score.total_answers}[/b].")
            r.append("")

        if user.fiorygi:
            r.append(f"Ha [b]{user.fiorygi}[/b].")
            r.append("")

        await data.reply("\n".join(r))



from .play import PlayCommand


class YoutubeCommand(PlayCommand):
    name: str = "youtube"

    aliases = ["yt"]

    description: str = "Cerca un video su YouTube e lo aggiunge alla coda della chat vocale."

    syntax = "{ricerca}"

    async def get_urls(self, args):
        return [f"ytsearch:{args.joined()}"]



from royalnet.constellation.api import *


class ApiDiscordCvStar(ApiStar):
    path = "/api/discord/cv/v1"

    summary = "Get the members status of a single Discord guild. Equivalent to calling /cv in a chat."

    tags = ["discord"]

    async def api(self, data: ApiData) -> dict:
        response = await self.interface.call_herald_event("discord", "discord_cv")
        return response



from typing import *
import royalnet.commands as rc
from ..version import semantic


class RoyalpackCommand(rc.Command):
    name: str = "royalpack"

    description: str = "Visualizza la versione attuale di Royalpack."

    syntax: str = ""

    async def run(self, args: rc.CommandArgs, data: rc.CommandData) -> None:
        if __debug__:
            message = f"ℹ️ Royalpack [url=https://github.com/Steffo99/royalpack/]Unreleased[/url]\n"
        else:
            message = f"ℹ️ Royalpack [url=https://github.com/Steffo99/royalpack/releases/tag/{semantic}]{semantic}[/url]\n"
        if "69" in semantic:
            message += "(Nice.)"
        await data.reply(message)



from .lazyplay import LazyplayCommand


class LazyyoutubeCommand(LazyplayCommand):
    name: str = "lazyyoutube"

    aliases = ["lyt"]

    description: str = "Cerca un video su YouTube e lo aggiunge (lazy) alla coda della chat vocale."

    syntax = "{ricerca}"

    async def get_urls(self, args):
        return [f"ytsearch:{args.joined()}"]



from typing import *
import logging
import asyncio
import datetime
import royalnet.commands as rc
from royalnet.utils import asyncify
from ..tables import Cvstats


log = logging.getLogger(__name__)


class CvstatsCommand(rc.Command):
    name: str = "cvstats"

    description: str = ""

    syntax: str = ""

    def __init__(self, interface: rc.CommandInterface):
        super().__init__(interface)
        if self.interface.name == "discord":
            self.loop.create_task(self._updater(1800))

    def _is_ryg_member(self, member: dict):
        for role in member["roles"]:
            if role["id"] == self.interface.config["Cv"]["displayed_role_id"]:
                return True
        return False

    async def _update(self, db_session):
        log.info(f"Gathering Cvstats...")
        while True:
            try:
                response: Dict[str, Any] = await self.interface.call_herald_event("discord", "discord_cv")
            except rc.ConfigurationError:
                await asyncio.sleep(10)
                continue
            else:
                break

        users_total = 0
        members_total = 0
        users_online = 0
        members_online = 0
        users_connected = 0
        members_connected = 0
        users_playing = 0
        members_playing = 0

        # noinspection PyUnboundLocalVariable
        for m in response["guild"]["members"]:
            users_total += 1
            if self._is_ryg_member(m):
                members_total += 1

            if m["status"]["main"] != "offline" and m["status"]["main"] != "idle":
                users_online += 1
                if self._is_ryg_member(m):
                    members_online += 1

                if m["voice"] is not None and not m["voice"]["afk"]:
                    users_connected += 1
                    if self._is_ryg_member(m):
                        members_connected += 1

                for mact in m["activities"]:
                    if mact.get("type") == 0:
                        users_playing += 1
                        if self._is_ryg_member(m):
                            members_playing += 1

        assert users_online >= members_online
        assert users_online >= users_connected
        assert users_online >= users_playing
        assert members_online >= members_connected
        assert members_online >= members_playing

        log.debug(f"Total users: {users_total}")
        log.debug(f"Total members: {members_total}")
        log.debug(f"Online users: {users_online}")
        log.debug(f"Online members: {members_online}")
        log.debug(f"Connected users: {users_connected}")
        log.debug(f"Connected members: {members_connected}")
        log.debug(f"Playing users: {users_playing}")
        log.debug(f"Playing members: {members_playing}")

        CvstatsT = self.alchemy.get(Cvstats)

        cvstats = CvstatsT(
            timestamp=datetime.datetime.now(),
            users_total=users_total,
            members_total=members_total,
            users_online=users_online,
            members_online=members_online,
            users_connected=users_connected,
            members_connected=members_connected,
            users_playing=users_playing,
            members_playing=members_playing
        )

        log.debug("Saving to database...")
        db_session.add(cvstats)
        await asyncify(db_session.commit)
        log.debug("Done!")

    async def _updater(self, period: int):
        log.info(f"Started updater with {period}s period")
        while True:
            log.info(f"Updating...")
            session = self.alchemy.Session()
            await self._update(session)
            session.close()
            log.info(f"Sleeping for {period}s")
            await asyncio.sleep(period)

    async def run(self, args: rc.CommandArgs, data: rc.CommandData) -> None:
        CvstatsT = self.alchemy.get(Cvstats)

        cvstats = data.session.query(CvstatsT).order_by(CvstatsT.timestamp.desc()).first()

        message = [
            f"ℹ️ [b]Statistiche[/b]",
            f"Ultimo aggiornamento: [b]{cvstats.timestamp.strftime('%Y-%m-%d %H:%M')}[/b]",
            f"Utenti totali: [b]{cvstats.users_total}[/b]",
            f"Membri totali: [b]{cvstats.members_total}[/b]",
            f"Utenti online: [b]{cvstats.users_online}[/b]",
            f"Membri online: [b]{cvstats.members_online}[/b]",
            f"Utenti connessi: [b]{cvstats.users_connected}[/b]",
            f"Membri connessi: [b]{cvstats.members_connected}[/b]",
            f"Utenti in gioco: [b]{cvstats.users_playing}[/b]",
            f"Membri in gioco: [b]{cvstats.members_playing}[/b]"
        ]

        await data.reply("\n".join(message))



import discord
from royalnet.commands import *


class SummonCommand(Command):
    name: str = "summon"

    aliases = ["cv"]

    description: str = "Evoca il bot in un canale vocale."

    syntax: str = "[nomecanale]"

    async def run(self, args: CommandArgs, data: CommandData) -> None:
        channel_name = args.joined()
        if self.interface.name == "discord":
            message: discord.Message = data.message
            guild_id = message.guild.id
            user_id = message.author.id
        else:
            guild_id = None
            user_id = None
        response = await self.interface.call_herald_event("discord", "discord_summon",
                                                          channel_name=channel_name, guild_id=guild_id, user_id=user_id)
        if self.interface.name == "discord":
            await data.reply(f"✅ Mi sono connesso in <#{response['channel']['id']}>!")
        else:
            await data.reply(f"✅ Mi sono connesso in [b]#{response['channel']['name']}[/b]!")



import aiohttp
import asyncio
import datetime
import logging
import dateparser
from royalnet.commands import *
from royalnet.serf.telegram.escape import escape


log = logging.getLogger(__name__)


class PeertubeUpdatesCommand(Command):
    name: str = "peertubeupdates"

    description: str = "Guarda quando è uscito l'ultimo video su PeerTube."

    aliases = ["ptu"]

    _ready = asyncio.Event()

    _latest_date: datetime.datetime = None

    def __init__(self, interface: CommandInterface):
        super().__init__(interface)
        if self.interface.name == "telegram":
            self.loop.create_task(self._ready_up())
            self.loop.create_task(self._update())

    async def _get_json(self):
        log.debug("Getting jsonfeed")
        async with aiohttp.ClientSession() as session:
            async with session.get(self.config["Peertube"]["instance_url"] +
                                   "/feeds/videos.json?sort=-publishedAt&filter=local") as response:
                log.debug("Parsing jsonfeed")
                if response.status != 200:
                    raise ExternalError("Peertube is unavailable")
                j = await response.json()
                log.debug("Jsonfeed parsed successfully")
        return j

    async def _send(self, message):
        client = self.interface.bot.client
        await self.interface.bot.safe_api_call(client.send_message,
                                               chat_id=self.config["Telegram"]["main_group_id"],
                                               text=escape(message),
                                               parse_mode="HTML",
                                               disable_webpage_preview=True)

    async def _ready_up(self):
        j = await self._get_json()
        if j["version"] != "https://jsonfeed.org/version/1":
            raise ConfigurationError("url is not a jsonfeed")
        videos = j["items"]
        for video in reversed(videos):
            date_modified = dateparser.parse(video["date_modified"])
            if self._latest_date is None or date_modified > self._latest_date:
                log.debug(f"Found newer video: {date_modified}")
                self._latest_date = date_modified
        self._ready.set()

    async def _update(self):
        await self._ready.wait()
        while True:
            j = await self._get_json()
            videos = j["items"]
            for video in reversed(videos):
                date_modified = dateparser.parse(video["date_modified"])
                if date_modified > self._latest_date:
                    log.debug(f"Found newer video: {date_modified}")
                    self._latest_date = date_modified
                    await self._send(f"🆕 Nuovo video su RoyalTube!\n"
                                     f"[b]{video['title']}[/b]\n"
                                     f"{video['url']}")
            await asyncio.sleep(self.config["Peertube"]["feed_update_timeout"])

    async def run(self, args: CommandArgs, data: CommandData) -> None:
        if self.interface.name != "telegram":
            raise UnsupportedError()
        await data.reply(f"ℹ️ Ultimo video caricato il: [b]{self._latest_date.isoformat()}[/b]")



from royalnet.constellation.api import *
from royalnet.utils import *
from ..tables import *


class ApiDiarioGetStar(ApiStar):
    path = "/api/diario/get/v1"

    summary = "Get the diario entry with a specific id."

    parameters = {
        "id": "The id of the diario entry to get."
    }

    tags = ["diario"]

    async def api(self, data: ApiData) -> dict:
        try:
            diario_id = int(data["id"])
        except ValueError:
            raise InvalidParameterError("'id' is not a valid int.")
        entry: Diario = await asyncify(data.session.query(self.alchemy.get(Diario)).get, diario_id)
        if entry is None:
            raise NotFoundError("No such diario entry.")
        return entry.json()



import royalnet
from royalnet.commands import *
from royalnet.backpack.tables import *


class EvalCommand(Command):
    # oh god if there is a security vulnerability
    name: str = "eval"

    description: str = "Esegui una espressione Python... se sei Steffo."

    syntax: str = "{espressione}"

    async def run(self, args: CommandArgs, data: CommandData) -> None:
        user: User = await data.get_author(error_if_none=True)
        if user.role != "Admin":
            raise CommandError("Non sei autorizzato a eseguire codice arbitrario!\n"
                               "(Sarebbe un po' pericoloso se te lo lasciassi eseguire, non trovi?)")
        try:
            result = eval(args.joined(require_at_least=1))
        except Exception as e:
            raise CommandError(f"Eval fallito: {e}")
        await data.reply(repr(result))



# Imports go here!
from .ciaoruozi import CiaoruoziCommand
from .color import ColorCommand
from .cv import CvCommand
from .diario import DiarioCommand
from .rage import RageCommand
from .reminder import ReminderCommand
from .ship import ShipCommand
from .smecds import SmecdsCommand
from .videochannel import VideochannelCommand
from .pause import PauseCommand
from .play import PlayCommand
from .queue import QueueCommand
from .skip import SkipCommand
from .summon import SummonCommand
from .youtube import YoutubeCommand
from .soundcloud import SoundcloudCommand
from .emojify import EmojifyCommand
from .leagueoflegends import LeagueoflegendsCommand
from .diarioquote import DiarioquoteCommand
from .peertubeupdates import PeertubeUpdatesCommand
from .googlevideo import GooglevideoCommand
from .yahoovideo import YahoovideoCommand
from .userinfo import UserinfoCommand
from .spell import SpellCommand
from .ahnonlosoio import AhnonlosoioCommand
from .eat import EatCommand
from .pmots import PmotsCommand
from .peertube import PeertubeCommand
from .funkwhale import FunkwhaleCommand
from .eval import EvalCommand
from .exec import ExecCommand
from .trivia import TriviaCommand
from .steampowered import SteampoweredCommand
from .steammatch import SteammatchCommand
from .dota import DotaCommand
from .magickfiorygi import MagickfiorygiCommand
from .brawlhalla import BrawlhallaCommand
from .diarioshuffle import DiarioshuffleCommand
from .funkwhaleplaylist import FunkwhaleplaylistCommand
from .voicestatus import VoicestatusCommand
from .playmode import PlaymodeCommand
from .lazyplay import LazyplayCommand
from .lazyfunkwhale import LazyfunkwhaleCommand
from .lazyfunkwhaleplaylist import LazyfunkwhaleplaylistCommand
from .lazygooglevideo import LazygooglevideoCommand
from .lazypeertube import LazypeertubeCommand
from .lazysoundcloud import LazysoundcloudCommand
from .lazyyahoovideo import LazyyahoovideoCommand
from .lazyyoutube import LazyyoutubeCommand
from .funkwhalealbum import FunkwhalealbumCommand
from .lazyfunkwhalealbum import LazyfunkwhalealbumCommand
from .matchmaking import MatchmakingCommand
from .cvstats import CvstatsCommand
from .elevatormusic import ElevatormusicCommand
from .royalpack import RoyalpackCommand

# Enter the commands of your Pack here!
available_commands = [
    CiaoruoziCommand,
    ColorCommand,
    CvCommand,
    DiarioCommand,
    RageCommand,
    ReminderCommand,
    ShipCommand,
    SmecdsCommand,
    VideochannelCommand,
    PauseCommand,
    PlayCommand,
    QueueCommand,
    SkipCommand,
    SummonCommand,
    YoutubeCommand,
    SoundcloudCommand,
    EmojifyCommand,
    LeagueoflegendsCommand,
    DiarioquoteCommand,
    PeertubeUpdatesCommand,
    GooglevideoCommand,
    YahoovideoCommand,
    UserinfoCommand,
    SpellCommand,
    AhnonlosoioCommand,
    EatCommand,
    PmotsCommand,
    PeertubeCommand,
    EvalCommand,
    ExecCommand,
    FunkwhaleCommand,
    TriviaCommand,
    SteampoweredCommand,
    SteammatchCommand,
    DotaCommand,
    MagickfiorygiCommand,
    BrawlhallaCommand,
    DiarioshuffleCommand,
    FunkwhaleplaylistCommand,
    VoicestatusCommand,
    PlaymodeCommand,
    LazyplayCommand,
    LazyfunkwhaleCommand,
    LazyfunkwhaleplaylistCommand,
    LazygooglevideoCommand,
    LazypeertubeCommand,
    LazysoundcloudCommand,
    LazyyahoovideoCommand,
    LazyyoutubeCommand,
    FunkwhalealbumCommand,
    LazyfunkwhalealbumCommand,
    MatchmakingCommand,
    CvstatsCommand,
    ElevatormusicCommand,
    RoyalpackCommand,
]

# Don't change this, it should automatically generate __all__
__all__ = [command.__name__ for command in available_commands]



from typing import *
from royalnet.utils import *
from royalnet.constellation.api import *
from ..tables import Poll
import uuid


class ApiPollsList(ApiStar):
    path = "/api/polls/list/v1"

    summary = "Get a list of all polls."

    requires_auth = True

    tags = ["polls"]

    async def api(self, data: ApiData) -> JSON:
        PollT = self.alchemy.get(Poll)

        polls: List[Poll] = await asyncify(data.session.query(PollT).all)

        return list(map(lambda p: {
            "id": p.id,
            "question": p.question,
            "creator": p.creator.json(),
            "expires": p.expires.isoformat(),
            "created": p.created.isoformat(),
        }, polls))



from sqlalchemy import *
from sqlalchemy.orm import *
from sqlalchemy.ext.declarative import declared_attr
from sqlalchemy.dialects.postgresql import UUID


class Poll:
    __tablename__ = "polls"

    @declared_attr
    def id(self):
        return Column(UUID(as_uuid=True), primary_key=True)

    @declared_attr
    def question(self):
        return Column(String, nullable=False)

    @declared_attr
    def description(self):
        return Column(Text, nullable=False, server_default="")

    @declared_attr
    def creator_id(self):
        return Column(Integer, ForeignKey("users.uid"))

    @declared_attr
    def creator(self):
        return relationship("User", backref=backref("polls_created"))

    @declared_attr
    def expires(self):
        return Column(DateTime)

    @declared_attr
    def created(self):
        return Column(DateTime, nullable=False)

    def json(self):
        return {
            "id": self.id,
            "question": self.question,
            "description": self.description,
            "creator": self.creator.json(),
            "expires": self.expires.isoformat(),
            "created": self.created.isoformat(),
            "votes": map(
                lambda v: {
                    "caster": v.caster.json(),
                    "posted": v.posted.isoformat(),
                    "vote": v.vote.name
                },
                sorted(self.votes, key=lambda v: v.posted)
            ),
            "comments": map(
                lambda c: {
                    "id": c.id,
                    "comment": c.comment,
                    "creator": c.creator.json(),
                    "posted": c.posted.isoformat(),
                    "mood": c.mood.name,
                },
                sorted(self.comments, key=lambda c: c.posted)
            )
        }



from starlette.requests import Request
from starlette.responses import *
from royalnet.constellation import *
from royalnet.utils import *
from royalnet.backpack.tables import *
from ..tables import *
from royalnet.constellation.api import *


class ApiWikiListStar(ApiStar):
    path = "/api/wiki/list/v1"

    summary = "Get a list of available wiki pages."

    tags = ["wiki"]

    async def api(self, data: ApiData) -> JSON:
        pages: typing.List[WikiPage] = await asyncify(data.session.query(self.alchemy.get(WikiPage)).all)
        return [page.json_list() for page in sorted(pages, key=lambda p: p.title)]
